@@ -49,8 +49,13 @@ struct page_read {
 	/* Advance page_read to the next entry (including zero pagemaps) */
 	int (*advance)(struct page_read *pr);
 	void (*close)(struct page_read *);
+	void (*skip_pages)(struct page_read *, unsigned long len);
 	int (*sync)(struct page_read *pr);
 	int (*seek_pagemap)(struct page_read *pr, unsigned long vaddr);
+	void (*reset)(struct page_read *pr);
+
+	/* Whether or not pages can be read in PIE code */
+	bool pieok;
 
 	/* Private data of reader */
 	struct cr_img *pmi;
@@ -95,8 +100,11 @@ extern int open_page_read(int pid, struct page_read *, int pr_flags);
 extern int open_page_read_at(int dfd, int pid, struct page_read *pr,
 		int pr_flags);
 
+struct task_restore_args;
+
 int pagemap_enqueue_iovec(struct page_read *pr, void *buf,
 			      unsigned long len, struct list_head *to);
+int pagemap_render_iovec(struct list_head *from, struct task_restore_args *ta);
 
 extern int dedup_one_iovec(struct page_read *pr, unsigned long base,
 			   unsigned long len);
