@@ -19,6 +19,9 @@ enum sfd_type {
 	CGROUP_YARD,
 	USERNSD_SK,	/* Socket for usernsd */
 	NS_FD_OFF,	/* Node's net namespace fd */
+	TRANSPORT_FD_OFF, /* to transfer file descriptors */
+	RPC_SK_OFF,
+	FDSTORE_SK_OFF,
 
 	SERVICE_FD_MAX
 };
