@@ -11,12 +11,18 @@
 #include "util-vdso.h"
 
 extern struct vdso_symtable vdso_sym_rt;
+extern struct vdso_symtable vdso_compat_rt;
 
 extern int vdso_init(void);
 
 extern int parasite_fixup_vdso(struct parasite_ctl *ctl, pid_t pid,
 			       struct vm_area_list *vma_area_list);
 
+#ifdef CONFIG_COMPAT
+void compat_vdso_helper(struct vdso_symtable *native, int pipe_fd,
+		int err_fd, void *vdso_buf, size_t buf_size);
+#endif
+
 #else /* CONFIG_VDSO */
 
 #define vdso_init()						(0)
