@@ -781,6 +781,7 @@ struct unix_sk_info {
 	char *name_dir;
 	unsigned flags;
 	struct unix_sk_info *peer;
+	struct pprep_head peer_resolve; /* XXX : union with the above? */
 	struct file_desc d;
 	struct list_head connected; /* List of sockets, connected to me */
 	struct list_head node; /* To link in peer's connected list  */
@@ -1359,24 +1360,29 @@ static void unlink_stale(struct unix_sk_info *ui)
 	revert_unix_sk_cwd(&cwd_fd, &root_fd);
 }
 
-static int resolve_unix_peers(void *unused);
+static int resolve_unix_peer(struct unix_sk_info *ui);
+
+static int post_prepare_unix_sk(struct pprep_head *ph)
+{
+	struct unix_sk_info *ui;
+
+	ui = container_of(ph, struct unix_sk_info, peer_resolve);
+	if (ui->ue->peer && resolve_unix_peer(ui))
+		return -1;
+	if (ui->name)
+		unlink_stale(ui);
+	return 0;
+}
 
 static int collect_one_unixsk(void *o, ProtobufCMessage *base, struct cr_img *i)
 {
 	struct unix_sk_info *ui = o;
-	static bool post_queued = false;
 	char *uname, *prefix = "";
 	int ulen;
 
 	ui->ue = pb_msg(base, UnixSkEntry);
 	ui->name_dir = (void *)ui->ue->name_dir;
 
-	if (ui->ue->peer && !post_queued) {
-		post_queued = true;
-		if (add_post_prepare_cb(resolve_unix_peers, NULL))
-			return -1;
-	}
-
 	if (ui->ue->name.len) {
 		if (ui->ue->name.len > UNIX_PATH_MAX) {
 			pr_err("Bad unix name len %d\n", (int)ui->ue->name.len);
@@ -1384,8 +1390,6 @@ static int collect_one_unixsk(void *o, ProtobufCMessage *base, struct cr_img *i)
 		}
 
 		ui->name = (void *)ui->ue->name.data;
-
-		unlink_stale(ui);
 	} else
 		ui->name = NULL;
 
@@ -1422,6 +1426,11 @@ static int collect_one_unixsk(void *o, ProtobufCMessage *base, struct cr_img *i)
 		prefix, ulen, uname,
 		ui->name_dir ? ui->name_dir : "-");
 
+	if (ui->ue->peer || ui->name) {
+		ui->peer_resolve.actor = post_prepare_unix_sk;
+		add_post_prepare_cb(&ui->peer_resolve);
+	}
+
 	list_add_tail(&ui->list, &unix_sockets);
 	return file_desc_add(&ui->d, ui->ue->id, &unix_desc_ops);
 }
@@ -1460,51 +1469,37 @@ static void interconnected_pair(struct unix_sk_info *ui, struct unix_sk_info *pe
 	}
 }
 
-static int resolve_unix_peers(void *unused)
+static int resolve_unix_peer(struct unix_sk_info *ui)
 {
-	struct unix_sk_info *ui, *peer;
+	struct unix_sk_info *peer;
 
-	list_for_each_entry(ui, &unix_sockets, list) {
-		if (ui->peer)
-			continue;
-		if (!ui->ue->peer)
-			continue;
+	if (ui->peer)
+		goto out;
 
-		peer = find_unix_sk_by_ino(ui->ue->peer);
+	BUG_ON(!ui->ue->peer);
 
-		if (!peer) {
-			pr_err("FATAL: Peer %#x unresolved for %#x\n",
-					ui->ue->peer, ui->ue->ino);
-			return -1;
-		}
-
-		set_peer(ui, peer);
-		if (!peer->queuer)
-			peer->queuer = ui->ue->ino;
-		if (ui == peer)
-			/* socket connected to self %) */
-			continue;
-		if (peer->ue->peer != ui->ue->ino)
-			continue;
-
-		set_peer(peer, ui);
-
-		/* socketpair or interconnected sockets */
-		interconnected_pair(ui, peer);
+	peer = find_unix_sk_by_ino(ui->ue->peer);
+	if (!peer) {
+		pr_err("FATAL: Peer %#x unresolved for %#x\n",
+				ui->ue->peer, ui->ue->ino);
+		return -1;
 	}
 
-	pr_info("Unix sockets:\n");
-	list_for_each_entry(ui, &unix_sockets, list) {
-		struct fdinfo_list_entry *fle;
-
-		pr_info("\t%#x -> %#x (%#x) flags %#x\n", ui->ue->ino, ui->ue->peer,
-				ui->peer ? ui->peer->ue->ino : 0, ui->flags);
-		list_for_each_entry(fle, &ui->d.fd_info_head, desc_list)
-			pr_info("\t\tfd %d in pid %d\n",
-					fle->fe->fd, fle->pid);
-
-	}
+	set_peer(ui, peer);
+	if (!peer->queuer)
+		peer->queuer = ui->ue->ino;
+	if (ui == peer)
+		/* socket connected to self %) */
+		goto out;
+	if (peer->ue->peer != ui->ue->ino)
+		goto out;
 
+	pr_info("Connected %#x -> %#x (%#x) flags %#x\n",
+			ui->ue->ino, ui->ue->peer, peer->ue->ino, ui->flags);
+	set_peer(peer, ui);
+	/* socketpair or interconnected sockets */
+	interconnected_pair(ui, peer);
+out:
 	return 0;
 }
 
