@@ -1,5 +1,3 @@
-#define _GNU_SOURCE
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +32,7 @@ TEST_OPTION(dirname, string, "directory name", 1);
 #define INDIRECT_MNT_DIR		"mnt"
 
 int autofs_dev;
+task_waiter_t t;
 
 static char *xvstrcat(char *str, const char *fmt, va_list args)
 {
@@ -221,18 +220,21 @@ static int check_fd(struct autofs_params *p)
 		ret++;
 	}
 	if (st.st_size != p->fd_stat.st_size) {
-		pr_err("%s: st_size differs: %ld != %ld\n", p->mountpoint,
-				st.st_size, p->fd_stat.st_size);
+		pr_err("%s: st_size differs: %lld != %lld\n", p->mountpoint,
+				(long long)st.st_size,
+				(long long)p->fd_stat.st_size);
 		ret++;
 	}
 	if (st.st_blksize != p->fd_stat.st_blksize) {
-		pr_err("%s: st_blksize differs %ld != %ld:\n", p->mountpoint,
-				(long)st.st_blksize, (long)p->fd_stat.st_blksize);
+		pr_err("%s: st_blksize differs %lld != %lld:\n", p->mountpoint,
+				(long long)st.st_blksize,
+				(long long)p->fd_stat.st_blksize);
 		ret++;
 	}
 	if (st.st_blocks != p->fd_stat.st_blocks) {
-		pr_err("%s: st_blocks differs: %ld != %ld\n", p->mountpoint,
-				st.st_blocks, p->fd_stat.st_blocks);
+		pr_err("%s: st_blocks differs: %lld != %lld\n", p->mountpoint,
+				(long long)st.st_blocks,
+				(long long)p->fd_stat.st_blocks);
 		ret++;
 	}
 
@@ -568,6 +570,7 @@ static int automountd(struct autofs_params *p, int control_fd)
 		goto err;
 	}
 	close(control_fd);
+	task_waiter_complete(&t, getpid());
 	return automountd_loop(pipes[0], autofs_path, p);
 
 err:
@@ -599,6 +602,7 @@ static int start_automounter(struct autofs_params *p)
 			close(control_fd[0]);
 			exit(automountd(p, control_fd[1]));
 	}
+	task_waiter_wait4(&t, pid);
 	p->pid = pid;
 
 	close(control_fd[1]);
@@ -877,6 +881,8 @@ int main(int argc, char **argv)
 
 	test_init(argc, argv);
 
+	task_waiter_init(&t);
+
 	if (mkdir(dirname, 0777) < 0) {
 		pr_perror("failed to create %s directory", dirname);
 		return -1;
